require_relative 'playable'
require_relative 'treasure_trove'

module StudioGameJB
  class Player
    include Playable

    attr_reader :name
    attr_accessor :health

    def initialize(name, health = 100)
      @name = name.capitalize
      @health = health
      @found_treasures = Hash.new 0
    end

    def <=>(other_player)
      other_player.score <=> score
    end

    def name=(new_name)
      @name = new_name.capitalize
    end

    def found_treasure(treasure)
      @found_treasures[treasure.name] += treasure.points
      puts "#{@name} found a #{treasure.name} worth #{treasure.points} points."
      puts "#{@name}'s treasures: #{@found_treasures}"
    end

    def points
      @found_treasures.values.reduce 0, :+
    end

    def score
      @health + points
    end

    def each_found_treasure
      @found_treasures.each do |name, points|
        yield Treasure.new(name, points)
      end
    end

    def to_s
      "I'm #{@name} with health = #{@health}, points = #{points}, and score = #{score}."
    end

    def print_name_and_health
      "#{@name} (#{@health})"
    end
  end
end
